__all__ = [
    '__title__',
    '__package_name__',
    '__version__',
    '__summary__',
    '__keywords__',
    '__author__',
    '__email__',
    '__author__',
    '__license__',
    '__url__',
    '__copyright__',
]

__title__ = 'Xonotic Map Manager'
__package_name__ = 'xmm'
__version__ = '0.8.0'
__summary__ = 'A package manager for Xonotic.'
__keywords__ = ['xonotic', 'xonotic-servers', 'xonotic-maps', 'gaming', 'cli']
__author__ = 'Tyler Mulligan'
__email__ = 'z@xnz.me'
__license__ = 'MIT'
__url__ = 'https://www.github.com/z/xonotic-map-manager'
__copyright__ = 'Copyright 2016 Tyler Mulligan'
