from .__about__ import (__title__, __package_name__, __version__, __summary__, __url__,
                        __keywords__, __email__, __author__, __license__, __copyright__)

from . import config
from . import logger

__all__ = [
    'config',
    'logger',
]
